﻿
; ### IP-Frame vom Server gesendet mit unterschiedlicher Länge ###
; # Start-Bytes: $55, $AA
; # Status der Ausgänge:
;   Für die Übertragung der aktuellen Ausgangszustände genügt 1 Byte.
;   Es folgt ein weiteres Byte für die Übertragung der Automatikzustände
; # Anzahl der nun folgenden Aufgaben: 2 Bytes (max. 1750 = 250 x 7 Bytes)
; # Aufgaben (Schaltzeiten), pro Aufgabe 7 Bytes:
;   Indexnummer der Liste 1 Byte. Enthält ein Eintrag in der Liste mehrere Wochentage, wird für jeden Wochentag 
;   eine eigene Aufgabe eingetragen. Es können also bis zu 7 Aufgaben die gleiche Indexnummer haben.
;   Die Wochenschaltuhr kann jede Minute einen Schaltvorgang auslösen. In einer Woche sind das 10080 Minuten. 
;   Als Speicher genügt für diesen Wert eine Integervariable oder 14 Bit. 
;   Jeder Ausgang kann zu jeder Schaltzeit entweder ein- oder ausgeschaltet, oder unverändert gelassen werden.
;   Als Speicher werden 8 Bit für die einzuschaltenden, und 8 Bit für die auszuschaltenden Ausgänge benötigt.
;   Das "Deaktiviert-Bit" wird in einem weiteren Byte übertragen. Auserdem ist darin noch Platz für Erweiterungen.
; # Checksumme:  XOR über alle Byte des Frame
;
; Frame: $55, $AA, Ausgangsstatus, Automatic, Aufgaben-L, Aufgaben-H, *Index, *Zeit-L, *Zeit-H, *Ausgänge-Ein, *Ausgänge-Aus, *Ausgänge-Impuls, *Deaktiv-Löschen, Checksumme   
;
; *SBuffer, $55)      ; Start
; *SBuffer+1, $AA)    ; Kennung $AA = Server->Client  
; *SBuffer+2, $0F)    ; Ausgangsstatus (8 Bit)
; *SBuffer+3, $0F)    ; Automatikstatus (8 Bit)
; *SBuffer+4, $01)    ; Aufgaben-L
; *SBuffer+5, $00)    ; Aufgaben-H
; *SBuffer+6, $00)    ; Index
; *SBuffer+7, $58)    ; Zeit-L  (Mo, 10:00)
; *SBuffer+8, $02)    ; Zeit-H
; *SBuffer+9, $20)    ; Ausgänge ein  (Ausgang 6 ein) 
; *SBuffer+10, $01)   ; Ausgänge aus  (Ausgang 1 aus)
; *SBuffer+11, $00)   ; Ausgänge Monoflop-Verhalten
; *SBuffer+12, $00)   ; Deaktiviert, Löschen nach Ausführung
; *SBuffer+13, $89)   ; Checksumme


; ### IP-Frame vom Server empfangen hat immer 16 Byte ###
; Alle Bytes, die für die jeweilige Übertragung nicht erforderlich sind, werden auf $00 gesetzt 
;
; *EBuffer, $55)      ; Start
; *EBuffer+1, $AA)    ; Kennung  $AB = Global-Request, $AC = Event-Set-Frame, $AD = Output-Status-Frame, $AE = Event-Delete-Frame  
; *EBuffer+2, $0F)    ; Ausgangsstatus (8 Bit)
; *EBuffer+3, $0F)    ; Automatikstatus (8 Bit)
; *EBuffer+4, $00)    ; Index
; *EBuffer+5, $58)    ; Zeit-L  (Mo, 10:00)
; *EBuffer+6, $02)    ; Zeit-H
; *EBuffer+7, $20)    ; Ausgänge ein  (Ausgang 6 ein) 
; *EBuffer+8, $01)    ; Ausgänge aus  (Ausgang 1 aus)
; *EBuffer+9, $00)    ; Ausgänge Monoflop-Verhalten
; *EBuffer+10, $00)   ; Deaktiviert, Löschen nach Ausführung
; *EBuffer+11, $89)   ; Sekunden
; *EBuffer+12, $00)   ; Minuten
; *EBuffer+13, $00)   ; Stunden
; *EBuffer+14, $00)   ; Wochentag
; *EBuffer+15, $89)   ; Checksumme
;
; Frame: $55, FrameKennung, AusgangsStatus, AutomaticStatus, *Nummer, *Zeit-L, *Zeit-H, *Ausgänge-Ein, *Ausgänge-Aus, *Ausgänge-Impuls, *Deaktiv-Löschen, Checksumme   

; Hardware:
; https://www.mikrocontroller.net/topic/404435

EnableExplicit

Structure ag
  Index.a
  Zeit.l
  AusgangEin.a
  AusgangAus.a
  AusgangImpuls.a
  Flags.a
EndStructure

Global *SBuffer = AllocateMemory(1800)
Global *EBuffer = AllocateMemory(16)
Global Dim Aufgabe.ag(250) ; in diesem Array werden zur Laufzeit ständig alle Schaltaufgaben vorgehalten und mit der aktuellen Zeit verglichen
Global Impulsdauer.l, Ausgaenge.a, Automatic.a, aNbr.i, ClientID, ImpStor.a

Define Event, Sevent, Port.l

Declare SetOutput()
Declare SetAutomatic()
Declare LoadEvents()

If InitNetwork() = 0
  MessageRequester("Error", "Can't initialize the network !", 0)
  End
EndIf

; die globalen Einstellungen laden
If OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  PreferenceGroup("Global")
  Port = ReadPreferenceLong("Port", 6832)
  Ausgaenge = Val("%" + ReadPreferenceString("Ausgaenge", "00000000"))
  Automatic = Val("%" + ReadPreferenceString("Automatic", "00000000"))
  Impulsdauer = ReadPreferenceLong("Impulsdauer", 1000)
ClosePreferences()
Else
  MessageRequester("Error", "Can't open the file 'server.ini'.", #MB_ICONERROR)
  End
EndIf 


OpenWindow(0, 50, 50, 320, 290, "Wochenschaltuhr Server", #PB_Window_ScreenCentered | #PB_Window_MinimizeGadget)
EditorGadget(0, 10, 10, 300, 180, #PB_Editor_ReadOnly)
;
OptionGadget(22, 40, 200, 15, 15, "") ; Automatic
ButtonGadget(32, 40, 260, 15, 15, "") 
OptionGadget(2, 40, 220, 15, 15, "") ; Ausgang 1
TextGadget(12, 43, 240, 11, 15, "1")
OptionGadget(23, 60, 200, 15, 15, "") ; Automatic
ButtonGadget(33, 60, 260, 15, 15, "") 
OptionGadget(3, 60, 220, 15, 15, "") ; Ausgang 2
TextGadget(13, 63, 240, 11, 15, "2")
OptionGadget(24, 80, 200, 15, 15, "") ; Automatic
ButtonGadget(34, 80, 260, 15, 15, "") 
OptionGadget(4, 80, 220, 15, 15, "") ; Ausgang 3
TextGadget(14, 83, 240, 11, 15, "3")
OptionGadget(25, 100, 200, 15, 15, "") ; Automatic
ButtonGadget(35, 100, 260, 15, 15, "") 
OptionGadget(5, 100, 220, 15, 15, "") ; Ausgang 4
TextGadget(15, 103, 240, 11, 15, "4")
OptionGadget(26, 120, 200, 15, 15, "") ; Automatic
ButtonGadget(36, 120, 260, 15, 15, "") 
OptionGadget(6, 120, 220, 15, 15, "") ; Ausgang 5
TextGadget(16, 123, 240, 11, 15, "5")
OptionGadget(27, 140, 200, 15, 15, "") ; Automatic
ButtonGadget(37, 140, 260, 15, 15, "") 
OptionGadget(7, 140, 220, 15, 15, "") ; Ausgang 6
TextGadget(17, 143, 240, 11, 15, "6")
OptionGadget(28, 160, 200, 15, 15, "") ; Automatic
ButtonGadget(38, 160, 260, 15, 15, "") 
OptionGadget(8, 160, 220, 15, 15, "") ; Ausgang 7
TextGadget(18, 163, 240, 11, 15, "7")
OptionGadget(29, 180, 200, 15, 15, "") ; Automatic
ButtonGadget(39, 180, 260, 15, 15, "") 
OptionGadget(9, 180, 220, 15, 15, "")  ; Ausgang 8
TextGadget(19, 183, 240, 11, 15, "8")
;
TextGadget(11, 205, 200, 100, 15, "Automatic")
TextGadget(21, 205, 220, 100, 15, "Ausgänge")
ButtonGadget(1, 271, 189, 40, 20, "Clr") 
AddWindowTimer(0, 111, 333) ; periodic task 333ms
SetWindowPos_(WindowID(0),#HWND_TOPMOST,0,0,0,0,#SWP_NOMOVE|#SWP_NOSIZE) ; setzt das Fenster vor alle anderen Fenster auf dem Desktop

SetOutput()    ; die Ausgänge setzen
SetAutomatic() ; die Ausgänge dem Automatikbetrieb oder dem Handbetrieb zuordnen
LoadEvents()   ; alle Schaltaufgaben aus der ini-Datei in das strukturierte Array laden


Procedure VerifyCheckSum()
  ; prüft die Checksumme, bei OK kommt 1 zurück 
  Define temp.a, ii.a
  For ii = 0 To 14
    temp = temp ! PeekA(*EBuffer+ii) ; EXOR
  Next
  If temp = PeekA(*EBuffer+15) ; die Position der Checksumme im *EBuffer
    ProcedureReturn 1
  EndIf
  ProcedureReturn 0
EndProcedure  


Procedure LoadEvents()
  ; alle Aufgaben aus der ini-Datei in das strukturierte Array laden
  OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  ExaminePreferenceGroups()
  aNbr = 0
  While NextPreferenceGroup() ; Solange Gruppen existieren...
    If PreferenceGroupName() <> "Global"
      Aufgabe(aNbr)\Index = ReadPreferenceLong("Index", 0)
      Aufgabe(aNbr)\Zeit = ReadPreferenceLong("Zeit", 0) 
      Aufgabe(aNbr)\AusgangEin = Val("%" + ReadPreferenceString("AusgangEin", "00000000"))
      Aufgabe(aNbr)\AusgangAus = Val("%" + ReadPreferenceString("AusgangAus", "00000000"))
      Aufgabe(aNbr)\AusgangImpuls = Val("%" + ReadPreferenceString("AusgangImpuls", "00000000"))
      Aufgabe(aNbr)\Flags = Val("%" + ReadPreferenceString("Flags", "00000000"))
      aNbr + 1
    EndIf
  Wend
  ClosePreferences()
EndProcedure


; Frame: $55, $AA, Ausgangsstatus, Automatikstatus, Aufgaben-L, Aufgaben-H, *Index, *Zeit-L, *Zeit-H, *Ausg.Ein, *Ausg.Aus, *Ausg.Impuls, *Flägs, Checksumme   
Procedure SendGlobalFrame()
  Define cnt.i, tmp.a, ii.i
; LoadEvents()
  ; Frame vorbereiten
  PokeA(*SBuffer,   $55)        ; Start-1
  PokeA(*SBuffer+1, $AA)        ; Start-2
  PokeA(*SBuffer+2, Ausgaenge)  ; Ausgangsstatus (8 Bit)
  PokeA(*SBuffer+3, Automatic)  ; Automaticstatus (8 Bit)
  
  PokeA(*SBuffer+4, aNbr % 256)    ; Aufgaben-L
  PokeA(*SBuffer+5, aNbr / 256)    ; Aufgaben-H 
  cnt = 0
  While cnt < aNbr
    PokeA(*SBuffer+6+(cnt*7), Aufgabe(cnt)\Index)         ; Index
    PokeA(*SBuffer+7+(cnt*7), Aufgabe(cnt)\Zeit % 256)    ; Zeit-L  (Mo, 10:00)
    PokeA(*SBuffer+8+(cnt*7), Aufgabe(cnt)\Zeit / 256)    ; Zeit-H
    PokeA(*SBuffer+9+(cnt*7), Aufgabe(cnt)\AusgangEin)    ; Ausgänge ein  (Ausgang 6 ein) 
    PokeA(*SBuffer+10+(cnt*7), Aufgabe(cnt)\AusgangAus)    ; Ausgänge aus  (Ausgang 1 aus)
    PokeA(*SBuffer+11+(cnt*7), Aufgabe(cnt)\AusgangImpuls)  ; Ausgänge Impuls-Verhalten
    PokeA(*SBuffer+12+(cnt*7), Aufgabe(cnt)\Flags)        ; Deaktiviert, Löschen nach Ausführung
    cnt + 1
  Wend
  ; CheckSumme erzeugen
  tmp = 0
  For ii = 0 To 5+(cnt*7)
    tmp = tmp ! PeekA(*SBuffer+ii) ; bitweise XOR
  Next
  PokeA(*SBuffer+ii, tmp) ; die CheckSumme nach allen anderen Daten im Buffer eintragen
  SendNetworkData(ClientID, *SBuffer, ii+1)
EndProcedure


; Event-Delete-Frame: $55, $AE, $00, $00, index, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, Checksumme   
Procedure DeleteEvent(nbr.a)
  ; alle Gruppen, die zu dem Index gehören löschen
  OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  ExaminePreferenceGroups()
  While NextPreferenceGroup() ; Solange Gruppen existieren...
    If PreferenceGroupName() <> "Global"
     If ReadPreferenceLong("Index", 0) = nbr
       RemovePreferenceGroup(PreferenceGroupName())
     EndIf
    EndIf  
  Wend
  ClosePreferences()
EndProcedure


; Event-Set-Frame: $55, $AC, $00, $00, *Index, *Zeit-L, *Zeit-H, *Ausgänge-Ein, *Ausgänge-Aus, *Ausgänge-Impuls, *Deaktiv-Löschen, $00, $00, $00, $00, Checksumme   
Procedure SetNewEvent()
  ; Ein Event in die Preference-Datei eintragen
  Define ii.i, groupname.s
  Define find.a
  OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  ; freien Gruppennamen finden
  For ii = 1 To 250  
    find = #False
    ExaminePreferenceGroups()
    While NextPreferenceGroup() ; Solange Gruppen existieren...
      If PreferenceGroupName() <> "Global"
        If FindString(PreferenceGroupName(), Str(ii))
          find = #True
        EndIf  
      EndIf  
    Wend
    If find = #False : Break : EndIf
  Next 
  groupname = "Aufgabe " + Str(ii) ; erster freier Gruppenname
  PreferenceGroup(groupname)
  WritePreferenceLong("Index", PeekA(*EBuffer+4))
  WritePreferenceLong("Zeit", PeekU(*EBuffer+5))
  WritePreferenceString("AusgangEin", RSet(Bin(PeekA(*EBuffer+7)), 8, "0"))
  WritePreferenceString("AusgangAus", RSet(Bin(PeekA(*EBuffer+8)), 8, "0"))
  WritePreferenceString("AusgangImpuls", RSet(Bin(PeekA(*EBuffer+9)), 8, "0"))
  WritePreferenceString("Flags", RSet(Bin(PeekA(*EBuffer+10)), 8, "0"))
  ClosePreferences()
EndProcedure


Procedure SetOutput()
  ; die Ausgänge setzen
  If Ausgaenge & %00000001 : SetGadgetState(2, 1) : Else : SetGadgetState(2, 0) : EndIf
  If Ausgaenge & %00000010 : SetGadgetState(3, 1) : Else : SetGadgetState(3, 0) : EndIf
  If Ausgaenge & %00000100 : SetGadgetState(4, 1) : Else : SetGadgetState(4, 0) : EndIf
  If Ausgaenge & %00001000 : SetGadgetState(5, 1) : Else : SetGadgetState(5, 0) : EndIf
  If Ausgaenge & %00010000 : SetGadgetState(6, 1) : Else : SetGadgetState(6, 0) : EndIf
  If Ausgaenge & %00100000 : SetGadgetState(7, 1) : Else : SetGadgetState(7, 0) : EndIf
  If Ausgaenge & %01000000 : SetGadgetState(8, 1) : Else : SetGadgetState(8, 0) : EndIf
  If Ausgaenge & %10000000 : SetGadgetState(9, 1) : Else : SetGadgetState(9, 0) : EndIf
EndProcedure 


Procedure SetAutomatic()
  ; automatisch oder manuell die Ausgänge schalten
  If Automatic & %00000001 : SetGadgetState(22, 1) : Else : SetGadgetState(22, 0) : EndIf
  If Automatic & %00000010 : SetGadgetState(23, 1) : Else : SetGadgetState(23, 0) : EndIf
  If Automatic & %00000100 : SetGadgetState(24, 1) : Else : SetGadgetState(24, 0) : EndIf
  If Automatic & %00001000 : SetGadgetState(25, 1) : Else : SetGadgetState(25, 0) : EndIf
  If Automatic & %00010000 : SetGadgetState(26, 1) : Else : SetGadgetState(26, 0) : EndIf
  If Automatic & %00100000 : SetGadgetState(27, 1) : Else : SetGadgetState(27, 0) : EndIf
  If Automatic & %01000000 : SetGadgetState(28, 1) : Else : SetGadgetState(28, 0) : EndIf
  If Automatic & %10000000 : SetGadgetState(29, 1) : Else : SetGadgetState(29, 0) : EndIf
EndProcedure  


Procedure SaveStatus()
  OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  PreferenceGroup("Global")
  WritePreferenceString("Ausgaenge", RSet(Bin(Ausgaenge), 8, "0"))
  WritePreferenceString("Automatic", RSet(Bin(Automatic), 8, "0"))
  ClosePreferences()
  RemoveWindowTimer(0, 113)
EndProcedure


Procedure SavePort(prt)
  OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
  PreferenceGroup("Global")
  WritePreferenceLong("Port", prt)
  ClosePreferences()
  Delay(1000)
  Define Handle = WindowID(0) 
  ShowWindow_(Handle, #SW_NORMAL)
  SetForegroundWindow_(Handle)
  MessageRequester("Wochenschaltuhr Server", 
                   "Der Port wurde geändert." + #LF$ +
                   "Neuer Port: " + Str(prt) + #LF$ +
                   "Das Programm wird nun geschlossen." + #LF$ +
                   "Starten sie es danach wieder neu.", 0)
  End
EndProcedure


; Status-Frame: $55, $AD, Ausgaenge, Automatic, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, Checksumme   
Procedure StatusEvent()
  ; Ausgangsstatus und Automaticstatus sofort setzen und in die Preference-Datei eintragen
  Ausgaenge = PeekA(*EBuffer+2)
  Automatic = PeekA(*EBuffer+3)
  SetOutput()
  SetAutomatic()
  SaveStatus()
EndProcedure


Procedure RollingState(btn.a)
  ; manuelle Einstellung des Ausgangsstatus: An (1), Auto (2), Aus (3), Auto (0), ...
  ; Automatic Ausgaenge
  ; 0         0
  ; 0         1
  ; 1         0
  ; 1         1 
  RemoveWindowTimer(0, 113) ; zum Zeit-neu-setzen löschen 
  Define sel.a = %00000001 << btn
  If (Ausgaenge & sel)
    Ausgaenge = Ausgaenge & (~ sel) ; Bit zurücksetzen
    If (Automatic & sel)
      Automatic = Automatic & (~ sel) ; Bit zurücksetzen
    Else
      Automatic = Automatic | sel ; Bit setzen
    EndIf
  Else
    Ausgaenge = Ausgaenge | sel ; Bit setzen
  EndIf  
  AddWindowTimer(0, 113, 5000) ; nach 5s Status in der Ini-Datei sichern
EndProcedure


Procedure PeriodicTask()
  ; 3 x pro sek.
  SetOutput() : SetAutomatic() ; setzt die Ausgänge
  Static counter.i
  counter + 1
  If counter < 30 : ProcedureReturn : EndIf 
  counter = 0
  ; wird 6 x pro Minute aufgerufen 
  Define cnt.i, minwert.i, temp.a
  Static oldwert.i = -1
  ; die aktuelle Zeit genau so codieren, wie die Schaltzeiten (Anzahl der Minuten ab Mo 00:00 Uhr) 
  Select DayOfWeek(Date())
    Case 1 : minwert = 0 ; Mo
    Case 2 : minwert = 1440
    Case 3 : minwert = 2880
    Case 4 : minwert = 4320
    Case 5 : minwert = 5760
    Case 6 : minwert = 7200
    Case 0 : minwert = 8640
  EndSelect  
  minwert + (Hour(Date()) * 60)  
  minwert + Minute(Date())  
  ; alle Schaltzeiten mit der aktuellen Zeit vergleichen
  cnt = 0
  While cnt < aNbr
    If minwert = Aufgabe(cnt)\Zeit And oldwert <> minwert
      If Aufgabe(cnt)\Flags & %00000001 ; Deaktiviert-Flag auswerten
        oldwert = minwert ; nur ein Mal ausführen
        ; Einschaltvorgänge
        Ausgaenge = Ausgaenge | (Aufgabe(cnt)\AusgangEin & Automatic)
        ; Ausschaltvorgänge
;        Ausgaenge = Ausgaenge & ((~ Aufgabe(cnt)\AusgangAus) | (~ Automatic))
        Ausgaenge = Ausgaenge & (~ (Aufgabe(cnt)\AusgangAus & Automatic))
        ; geänderte Ausgänge in der Preference-Datei speichern
        OpenPreferences("server.ini", #PB_Preference_GroupSeparator)
        PreferenceGroup("Global")
        WritePreferenceString("Ausgaenge", RSet(Bin(Ausgaenge), 8, "0"))
        ClosePreferences()
        ; Ausgang nur für kurze Zeit umschalten (Impuls mit XOR-Verhalten ! )
        ImpStor = Aufgabe(cnt)\AusgangImpuls & Automatic
        Ausgaenge = Ausgaenge ! ImpStor
        AddWindowTimer(0, 112, Impulsdauer) ; Wert (in Millisekunden) aus der Preference-Datei
        ; Flags für Löschen der Aufgabe nach Ausführung auswerten
        If Aufgabe(cnt)\Flags & %00000010
          DeleteEvent(Aufgabe(cnt)\Index)
          LoadEvents()
        EndIf  
      EndIf  
    EndIf
    cnt + 1
  Wend  
EndProcedure


Procedure AddMsg(txt$)
  ; add an entry to the list and scroll, that it is visible
  txt$ = RTrim(txt$, Chr(10))
  txt$ = FormatDate("%hh:%ii:%ss", Date()) + " " + RTrim(txt$, Chr(13))
  AddGadgetItem (0, -1, txt$)
  SendMessage_(GadgetID(0),#EM_SCROLL, #SB_BOTTOM, 0)
EndProcedure 


If CreateNetworkServer(0, Port)
  AddMsg("Server started (Port " + Str(Port) + ")")
  Repeat
    Event = WaitWindowEvent(20)
    Select NetworkServerEvent() ;Sevent
      Case #PB_NetworkEvent_Connect
        If ClientID = 0 ; nur eine Verbindung z.Z. zulassen
          ClientID = EventClient()
          AddMsg("Client " + Str(ClientID) + " is connected")
        EndIf
      Case #PB_NetworkEvent_Data
        ReceiveNetworkData(ClientID, *EBuffer, 16)
        If PeekA(*EBuffer) = $55 ; dieses Byte muss immer vorhanden sein, wenn der Frame für den Server bestimmt ist
          If VerifyCheckSum() ; wenn die Checksumme stimmt
            Select PeekA(*EBuffer+1) ; Frame-Kennung auswerten - das zeigt, was der Client will
              Case $A8 ; der Frame überträgt die aktuelle PC-Zeit zum Server und ist bei der echten Hardware zum stellen der RTC nützlich  
                AddMsg("Client "+Str(ClientID) + " Time: "+RSet(Str(PeekA(*EBuffer+13)), 2, "0")+":"+RSet(Str(PeekA(*EBuffer+12)), 2, "0")+":"+RSet(Str(PeekA(*EBuffer+11)), 2, "0"))
                
              Case $A9 ; der Frame überträgt die neue IP-Adresse und den Port zum Server - hier wird nur der Port geändert  
                Port = PeekA(*EBuffer+14) * 256 + PeekA(*EBuffer+13)
                AddMsg("New Port is: " + Str(Port))
                SavePort(Port) ; das Programm wird hier geschlossen 
              Case $AB  ; Server soll alle Daten senden
                AddMsg("Client " + Str(ClientID) + " has send a global frame request")
                SendGlobalFrame()
              Case $AC  ; der Frame enthält ein neues Event, das der Server zukünftig berücksichtigen muss
                AddMsg("Client " + Str(ClientID) + " has send a event set frame")
                SetNewEvent()
                LoadEvents() ; Preference-Datei neu einlesen
              Case $AD  ; der Frame enthält Daten über den aktuellen Schalt- und Automatikzustand der Ausgänge 
                AddMsg("Client " + Str(ClientID) + " has send a status set frame")
                StatusEvent()
              Case $AE  ; der Frame enthält Informationen über ein Event, das gelöscht werden soll 
                AddMsg("Client " + Str(ClientID) + " has send a event delete frame")
                DeleteEvent(PeekA(*EBuffer+4))
                LoadEvents() ; Preference-Datei neu einlesen
              Case $AF  ; der Frame zeigt, dass der Client noch verbunden ist 
                AddMsg("Client " + Str(ClientID) + " keep alive")
                RemoveWindowTimer(0, 114) ; den Rausschmeiss-Timer reseten
                AddWindowTimer(0, 114, 11000) ; 11sek.
            EndSelect
          EndIf  
        Else
          Sevent = 0 : ClientID = 0
        EndIf
      Case #PB_NetworkEvent_Disconnect
        RemoveWindowTimer(0, 114) ; den Rausschmeiss-Timer löschen
        AddMsg("Client " + Str(ClientID) + " has closed the connection")
        Sevent = 0 : ClientID = 0
    EndSelect
    ;
    If Event = #PB_Event_Gadget
      Select EventGadget()
        Case 1 : ClearGadgetItems(0)
        Case 32 : RollingState(0)
        Case 33 : RollingState(1)
        Case 34 : RollingState(2)
        Case 35 : RollingState(3)
        Case 36 : RollingState(4)
        Case 37 : RollingState(5)
        Case 38 : RollingState(6)
        Case 39 : RollingState(7)
      EndSelect
    EndIf
    If Event = #PB_Event_Timer 
      Select EventTimer() 
        Case 111
          PeriodicTask() ; jede Sekunde die Procedure 3 x aufrufen
        Case 112
          Ausgaenge = Ausgaenge ! ImpStor ; die Ausgänge vom Timer nach einem Impuls zurücksetzen  
          SetOutput()
          RemoveWindowTimer(0, 112)
        Case 113  
          SaveStatus() ; alle Ausgangszustände in die ini-Datei schreiben sobald 5 Sek. lang keine Einstell-Button mehr angeklickt wurden
        Case 114
          CloseNetworkConnection(ClientID) ; die Verbindung schließen, wenn der Client sich nicht mehr meldet
          AddMsg("Server has closed the connection " + Str(ClientID))
          ClientID = 0
          RemoveWindowTimer(0, 114)
      EndSelect
    EndIf    
  Until Event = #PB_Event_CloseWindow 
  CloseNetworkServer(0)
Else
  MessageRequester("Error", "Can't create the server (port in use ?).", 0)
EndIf
  
End   

; IDE Options = PureBasic 5.70 LTS (Windows - x64)
; CursorPosition = 295
; FirstLine = 291
; Folding = ---
; EnableXP
; Executable = Server.exe
; Compiler = PureBasic 5.70 LTS (Windows - x86)